/*
 *  eXist Open Source Native XML Database
 *  Copyright (C) 2001-04 The eXist Project
 *  http://exist-db.org
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *  
 *  $Id$
 */
package org.exist.storage.index;

import java.nio.ByteBuffer;

import org.exist.storage.DBBroker;
import org.exist.storage.journal.LogException;
import org.exist.storage.txn.Txn;

/**
 * @author wolf
 *
 */
public class OverflowRemoveLoggable extends AbstractBFileLoggable {

    protected byte status;
    protected long pageNum;
    protected byte[] data;
    protected int length;
    protected long nextInChain;
    
    /**
     * 
     * 
     * @param status 
     * @param pageNum 
     * @param data 
     * @param length 
     * @param nextInChain 
     * @param fileId 
     * @param transaction 
     */
    public OverflowRemoveLoggable(byte fileId, Txn transaction, byte status, long pageNum, byte[] data, 
            int length, long nextInChain) {
        super(BFile.LOG_OVERFLOW_REMOVE, fileId, transaction);
        this.status = status;
        this.pageNum = pageNum;
        this.data = data;
        this.length = length;
        this.nextInChain = nextInChain;
    }

    /**
     * @param broker
     * @param transactionId
     */
    public OverflowRemoveLoggable(DBBroker broker, long transactionId) {
        super(broker, transactionId);
    }

    /* (non-Javadoc)
     * @see org.exist.storage.log.Loggable#write(java.nio.ByteBuffer)
     */
    public void write(ByteBuffer out) {
        System.out.println("Length = " + length);
        super.write(out);
        out.put(status);
        out.putInt((int) pageNum);
        out.putInt((int) nextInChain);
        out.putInt(length);
        out.put(data, 0, length);
    }

    /* (non-Javadoc)
     * @see org.exist.storage.log.Loggable#read(java.nio.ByteBuffer)
     */
    public void read(ByteBuffer in) {
        super.read(in);
        status = in.get();
        pageNum = in.getInt();
        nextInChain = in.getInt();
        length = in.getInt();
        data = new byte[length];
        in.get(data);
    }

    /* (non-Javadoc)
     * @see org.exist.storage.log.Loggable#getLogSize()
     */
    public int getLogSize() {
        return super.getLogSize() + 13 + length;
    }

    public void redo() throws LogException {
        getIndexFile().redoRemoveOverflow(this);
    }
    
    public void undo() throws LogException {
        getIndexFile().undoRemoveOverflow(this);
    }
    
    public String dump() {
        return super.dump() + " - remove overflow page " + pageNum;
    }
}
